#include <stdlib.h>
#include "mpdmonitor.h"

using namespace std;

const char *unknownStr = "<unknown>";

mpd_song *currSong = NULL;

int handleError(struct mpd_connection *c) {
    cout << "ERROR" << endl;
    assert(mpd_connection_get_error(c) != MPD_ERROR_SUCCESS);

    cerr << mpd_connection_get_error_message(c) << endl;
    mpd_connection_free(c);
    return -1;
}

void idle(mpd_connection *c) {
    /* Wait til something happens */
    mpd_idle idle = mpd_run_idle_mask(c, MPD_IDLE_PLAYER);
    if (idle == 0 && mpd_connection_get_error(c) != MPD_ERROR_SUCCESS)
        exit(handleError(c));
}

void notifySong(mpd_song *song) {
    if (notify_is_initted()) {
        stringstream ss;
        const char *title = mpd_song_get_tag(song, MPD_TAG_TITLE, 0);
        const char *album = mpd_song_get_tag(song, MPD_TAG_ALBUM, 0);
        const char *artist = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);

        ss << (title ? title : unknownStr) << endl
           << (album ? album : unknownStr) << endl
           << (artist ? artist : unknownStr);
             

        NotifyNotification *n = notify_notification_new("Currently playing",
                ss.str().c_str(), "", NULL);
        notify_notification_show(n, NULL);
    }
    else
        cerr << "Not showing notificatiion, libnotify not initted" << endl;
}

void printSong(mpd_song *song) {
    const char *title = mpd_song_get_tag(song, MPD_TAG_TITLE, 0);
    const char *album = mpd_song_get_tag(song, MPD_TAG_ALBUM, 0);
    const char *artist = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);

    cout << (title ? title : unknownStr) << endl;
    cout << (album ? album : unknownStr) << endl;
    cout << (artist ? artist : unknownStr) << endl;
}

void signalCallbackHandler(int signum) {
    if (currSong) {
        cout << "Song id: " << mpd_song_get_id(currSong) << endl;
        notifySong(currSong);
        printSong(currSong);
    }
}

int main (int argc, char **argv) {
    mpd_connection *conn = mpd_connection_new(NULL, 0, 30000);
    if (mpd_connection_get_error(conn) != MPD_ERROR_SUCCESS)
        return handleError(conn);

    if (!notify_init("mpdmonitor"))
        cerr << "error with libnotify init" << endl;

    signal(SIGUSR1, signalCallbackHandler);

    unsigned int songID = 0;
    while (true) {
        if (currSong)
            mpd_song_free(currSong);

        currSong = mpd_run_current_song(conn);
        if (!currSong)
        {
            idle(conn);
            continue;
        }

        unsigned int newSongID = mpd_song_get_id(currSong);
        if (songID != newSongID) {
            songID = newSongID;
            cout << "Song id: " << songID << endl;
            notifySong(currSong);
            printSong(currSong);
        }

        idle(conn);
    }

    mpd_connection_free(conn);
    notify_uninit();

    return 0;
}
